@@ -84,6 +84,7 @@ class IRBuilder(object):
     def __init__(self, block=None):
         self._block = block
         self._anchor = len(block.instructions) if block else 0
+        self.debug_metadata = None
 
     @property
     def block(self):
@@ -206,6 +207,8 @@ class IRBuilder(object):
         self.position_at_end(bbend)
 
     def _insert(self, instr):
+        if self.debug_metadata is not None and 'dbg' not in instr.metadata:
+            instr.metadata['dbg'] = self.debug_metadata
         self._block.instructions.insert(self._anchor, instr)
         self._anchor += 1
 
@@ -443,19 +446,21 @@ class IRBuilder(object):
         self._insert(al)
         return al
 
-    def load(self, ptr, name=''):
+    def load(self, ptr, name='', align=None):
         if not isinstance(ptr.type, types.PointerType):
             raise TypeError("cannot load from value of type %s (%r): not a pointer"
                             % (ptr.type, str(ptr)))
         ld = instructions.LoadInstr(self.block, ptr, name)
+        ld.align = align
         self._insert(ld)
         return ld
 
-    def store(self, value, ptr):
+    def store(self, value, ptr, align=None):
         if not isinstance(ptr.type, types.PointerType):
             raise TypeError("cannot store to value of type %s (%r): not a pointer"
                             % (ptr.type, str(ptr)))
         st = instructions.StoreInstr(self.block, value, ptr)
+        st.align = align
         self._insert(st)
         return st
 
